#!/usr/bin/python3
# coding: utf-8
import sys
import pyswf
import matplotlib.pyplot as plt
import datetime

def plot_prime_usage_per_day(swf_trace):
    plt.clf()  # preemptively clean pyplot's canvas
    # eight o'clock of the first day of the trace
    first_prime_time = pyswf.datetime.datetime(swf_trace.start_time.year, swf_trace.start_time.month, swf_trace.start_time.day, 8, 0, 0)
    # seconds elapsed until eight o'clock of the first trace day
    start_seconds =(first_prime_time-swf_trace.start_time).seconds
    for day in range((swf_trace.end_time-first_prime_time).days):
        print(f"Computing day {day}")
        # window of interest time in seconds wrt start of the trace
        woi_start = start_seconds + day*24*3600
        woi_end = start_seconds + day*24*3600 + 12*3600
        day_jobs = swf_trace.job_list[~(((swf_trace.job_list["submit_time"] < woi_start) & (swf_trace.job_list["end_time"] < woi_start)) | ((swf_trace.job_list["submit_time"] > woi_end) & (swf_trace.job_list["end_time"] > woi_end)))]
        time_steps = list(day_jobs["start_time"]) + list(day_jobs["end_time"]) + list(day_jobs["submit_time"]) + list(day_jobs["start_time"]-1) + list(day_jobs["end_time"]-1) + list(day_jobs["submit_time"]-1)
        time_steps = list(set(time_steps))  # remove duplicates
        time_steps.sort() 
        utilization = []
        requested = []
        for time in time_steps:
            active_jobs = day_jobs[(day_jobs["start_time"] <= time) & (day_jobs["end_time"] > time)]
            queueing_jobs = day_jobs[(day_jobs["submit_time"] <= time) & (day_jobs["start_time"] > time)]
            utilization.append(active_jobs["alloc_cpus"].sum())
            requested.append(queueing_jobs["alloc_cpus"].sum())
        plt.plot(time_steps, utilization, label="utilization")
        plt.plot(time_steps, requested, label="requested")
        #plt.axvline(start_seconds + day*24*3600, "g", label="8AM")
        #plt.axvline(start_seconds + day*24*3600 + 12*3600, "r", label="8PM")
        plt.xlim(woi_start,woi_end)
        plt.title(f"Used and Requested resources for day {day} during 9h - 18h\n{swf_trace.location}")
        plt.legend()
        plt.savefig(f"{'_'.join(swf_trace.location.lower().split())}_util_req_{day}.png")
        plt.clf()
        
if __name__ == "__main__":
    swf_trace = pyswf.SWFtrace.from_swf(sys.argv[1])
    swf_trace.job_list.insert(len(swf_trace.job_list.columns)-1, "start_time", swf_trace.job_list["submit_time"] + swf_trace.job_list["wait_time"])
    swf_trace.job_list.insert(len(swf_trace.job_list.columns)-1, "end_time", swf_trace.job_list["submit_time"] + swf_trace.job_list["wait_time"] + swf_trace.job_list["runtime"])
    plot_prime_usage_per_day(swf_trace)
